Package: ramr
Title: Detection of Rare Aberrantly Methylated Regions in Array and NGS
        Data
Version: 1.20.0
Authors@R: 
  person(given = "Oleksii",
    family = "Nikolaienko",
    role = c("aut", "cre"),
    email = "oleksii.nikolaienko@gmail.com",
    comment = c(ORCID = "0000-0002-5910-4934"))
Description: ramr is an R package for detection of epimutations (i.e.,
  infrequent aberrant DNA methylation events)
  in large data sets obtained by methylation profiling using array or
  high-throughput methylation sequencing. In addition, package
  provides functions to visualize found aberrantly methylated regions (AMRs),
  to generate sets of all possible regions to be used as reference sets for
  enrichment analysis, and to generate biologically relevant test data sets
  for performance evaluation of AMR/DMR search algorithms.
SystemRequirements: C++20, GNU make
NeedsCompilation: yes
Depends: R (>= 4.1)
Imports: methods, data.table, Seqinfo, GenomicRanges, IRanges,
        BiocGenerics, S4Vectors, Rcpp
LinkingTo: Rcpp
Suggests: RUnit, knitr, rmarkdown, ggplot2, gridExtra, annotatr, LOLA,
        org.Hs.eg.db, TxDb.Hsapiens.UCSC.hg19.knownGene, parallel,
        doParallel, foreach, doRNG, matrixStats, EnvStats, ExtDist,
        gamlss, gamlss.dist
License: Artistic-2.0
URL: https://github.com/BBCG/ramr
BugReports: https://github.com/BBCG/ramr/issues
Encoding: UTF-8
biocViews: DNAMethylation, DifferentialMethylation, Epigenetics,
        MethylationArray, MethylSeq
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ramr
git_branch: RELEASE_3_23
git_last_commit: 0e5de4b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-29 00:06:36 UTC; biocbuild
Author: Oleksii Nikolaienko [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5910-4934>)
Maintainer: Oleksii Nikolaienko <oleksii.nikolaienko@gmail.com>
