# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rcpp_compute_logp_beta <- function(data) {
    .Call(`_ramr_rcpp_compute_logp_beta`, data)
}

rcpp_compute_logp_beta_binom <- function(data) {
    .Call(`_ramr_rcpp_compute_logp_beta_binom`, data)
}

rcpp_compute_xiqr <- function(data) {
    .Call(`_ramr_rcpp_compute_xiqr`, data)
}

rcpp_create_granges_stranded_xiqr <- function(data, window, min_ncpg, min_width) {
    .Call(`_ramr_rcpp_create_granges_stranded_xiqr`, data, window, min_ncpg, min_width)
}

rcpp_create_granges_stranded_logp <- function(data, window, min_ncpg, min_width) {
    .Call(`_ramr_rcpp_create_granges_stranded_logp`, data, window, min_ncpg, min_width)
}

rcpp_create_granges_unstranded_xiqr <- function(data, window, min_ncpg, min_width) {
    .Call(`_ramr_rcpp_create_granges_unstranded_xiqr`, data, window, min_ncpg, min_width)
}

rcpp_create_granges_unstranded_logp <- function(data, window, min_ncpg, min_width) {
    .Call(`_ramr_rcpp_create_granges_unstranded_logp`, data, window, min_ncpg, min_width)
}

rcpp_filter_threshold_xiqr <- function(data, thrshld) {
    .Call(`_ramr_rcpp_filter_threshold_xiqr`, data, thrshld)
}

rcpp_filter_threshold_logp <- function(data, thrshld) {
    .Call(`_ramr_rcpp_filter_threshold_logp`, data, thrshld)
}

rcpp_fit_beta_mom <- function(data) {
    .Call(`_ramr_rcpp_fit_beta_mom`, data)
}

rcpp_fit_beta_amle <- function(data) {
    .Call(`_ramr_rcpp_fit_beta_amle`, data)
}

rcpp_fit_binom <- function(data) {
    .Call(`_ramr_rcpp_fit_binom`, data)
}

rcpp_generate_random_values <- function(data, ncol) {
    .Call(`_ramr_rcpp_generate_random_values`, data, ncol)
}

rcpp_get_iqr <- function(data) {
    .Call(`_ramr_rcpp_get_iqr`, data)
}

rcpp_get_meanvar_ari_equal <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_ari_equal`, data)
}

rcpp_get_meanvar_ari_invDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_ari_invDist`, data)
}

rcpp_get_meanvar_ari_sqrtInvDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_ari_sqrtInvDist`, data)
}

rcpp_get_meanvar_ari_logInvDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_ari_logInvDist`, data)
}

rcpp_get_meanvar_geo_equal <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_geo_equal`, data)
}

rcpp_get_meanvar_geo_invDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_geo_invDist`, data)
}

rcpp_get_meanvar_geo_sqrtInvDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_geo_sqrtInvDist`, data)
}

rcpp_get_meanvar_geo_logInvDist <- function(data) {
    .Call(`_ramr_rcpp_get_meanvar_geo_logInvDist`, data)
}

rcpp_extract_out <- function(data) {
    .Call(`_ramr_rcpp_extract_out`, data)
}

rcpp_extract_coef <- function(data) {
    .Call(`_ramr_rcpp_extract_coef`, data)
}

rcpp_prepare_data_identity <- function(seqnames, seqrunlens, start, strand, mcols, coverage, exclude_lower, exclude_upper, chunks) {
    .Call(`_ramr_rcpp_prepare_data_identity`, seqnames, seqrunlens, start, strand, mcols, coverage, exclude_lower, exclude_upper, chunks)
}

rcpp_prepare_data_linear <- function(seqnames, seqrunlens, start, strand, mcols, coverage, exclude_lower, exclude_upper, chunks) {
    .Call(`_ramr_rcpp_prepare_data_linear`, seqnames, seqrunlens, start, strand, mcols, coverage, exclude_lower, exclude_upper, chunks)
}

rcpp_test_omp <- function() {
    .Call(`_ramr_rcpp_test_omp`)
}

